require 'polygallery'

module Polygallery
  module ActsAsPolyphoto
    extend ActiveSupport::Concern
    module ClassMethods
      def acts_as_polyphoto(options={})
        defaults = self.polygallery_settings
        settings = defaults.deep_merge(options)

        attr_accessor :photo_to_upload, :polygallery_options,
                      :polyphoto_initialized
        attr_reader   :photo_remote_url
        
        init_attachment settings[:paperclip]
        init_associations settings

        before_validation do
          # self.polygallery_options = nil
          # self.polygallery_options = self.polygallery_settings
          # self.class.init_attachment self.polygallery_options
          if gallery.present?
            self.gallery_title = gallery.title if gallery_title.blank?
            self.galleryable ||= gallery.galleryable
            initialize_polyphoto
          end
          process_photo_to_upload
        end
        # before_save {
        #   # puts "Photo changes: #{self.changes.inspect}"
        #   puts 'SAVING PHOTO!' }

        after_initialize do
          self.polygallery_options ||= self.polygallery_settings
          self.initialize_polyphoto
        end

        include ActsAsPolyphoto::LocalInstanceMethods
      end

      def polygallery_settings
        DEFAULT_SETTINGS
      end

      def init_attachment(paperclip_settings)
        has_attached_file :photo,
          :styles => ->(a) {
              a.instance.paperclip_settings[:styles] },
          :default_url => paperclip_settings[:default_url]
        validations = polygallery_settings[:paperclip_validations]
        validates_attachment_content_type(:photo, :content_type =>
                   validations[:content_type]) if validations[:content_type]
        validates_attachment_presence(:photo) if validations[:presence]
      end

      def init_associations(settings=DEFAULT_SETTINGS)
        belongs_to settings[:association_names][:gallery],
          :class_name => settings[:associations][:gallery][:class_name],
          :foreign_key => :gallery_id
        belongs_to :galleryable, :polymorphic => true
      end

    end

    module LocalInstanceMethods
      def include_polygallery_settings(settings)
        self.polygallery_options = settings
        self.initialize_polyphoto
      end

      def initialize_polyphoto
        return if self.polyphoto_initialized.present?
        self.class.init_associations(self.polygallery_settings)
        self.init_attachment
        self.polyphoto_initialized = true
      end

      def polygallery_settings
        return self.polygallery_options if self.polygallery_options.present?
        if self.gallery_title.present?
          s = if self.galleryable.present?
                galleryable.class.send(:"#{self.gallery_title}_settings")
              elsif self.galleryable_type.present?
                Kernel.const_get(self.galleryable_type)
                  .send(:"#{self.gallery_title}_settings")
              end
          return s if s.present?
        elsif respond_to?(:gallery) && gallery.present?
          return gallery.polygallery_settings
        elsif self.gallery_id.present?
          g = Polygallery::Gallery.find(self.gallery_id)
          if g.present?
            self.gallery_title = g.title
            return self.polygallery_settings
          end
        end
        self.class.polygallery_settings
        # if gallery_title.present? && galleryable_type.present?
        #   galleryable_class = Kernel.const_get(galleryable_type)
        #   galleryable_class.send :"#{gallery_title}_settings"
        # elsif gallery_title.present?
        #   self.send(gallery_title.to_sym).polygallery_settings
        # elsif self.gallery.present? && gallery.polygallery_settings.present?
        #   self.gallery.polygallery_settings 
        # else
        #   DEFAULT_SETTINGS
        # end
      end

      def paperclip_settings
        self.polygallery_settings[:paperclip]
      end

      def process_photo_to_upload
        # As crazy as this looks, it's necessary to assign paperclip's
        # dynamic styles through an association.  It would be great if we
        # could find a better way, though...
        #
        # This part runs the second try.
        if (new_record? || photo_updated_at_changed?) && gallery.present? &&
            photo_to_upload.present? && File.exists?(photo_to_upload)
          # puts 'Running the swaparoo!'
          File.open(photo_to_upload, 'rb') {|h| self.photo = h }
          # self.photo_to_upload = nil
        end
        # This part runs the first try.
        if self.photo.present? && self.photo_to_upload.nil?
          # if self.new_record? || self.photo_file_name_changed?
          if self.photo.staged?
            # puts "#{photo_file_name} is staged!"
            if File.exists? self.photo.staged_path
              # puts "Setting photo to upload to #{photo_file_name}"
              self.photo_to_upload = self.photo.staged_path
              # self.photo = nil
            end
          end
        end
      end

      def init_attachment
        self.class.init_attachment paperclip_settings
      end

      def remote_url(style=nil)
        host = ActionController::Base.asset_host
        host = "http://#{host}" unless host.nil? || host =~ /^http/
        path = photo.url(style)
        url = (host||'') + (path||'')
        url unless url.blank?
      end
      def thumb_url; remote_url(:thumb) end

      def process_remote_uri(uri)
        require 'open-uri'
        require 'open_uri_redirections'
        open(uri, allow_redirections: :safe) {|r| r.base_uri.to_s }
      end
      def photo_remote_url=(url_value)
        self.photo = URI.parse process_remote_uri(url_value)
        @photo_remote_url = url_value
      end

    end
  end
end
